#!/usr/bin/perl -w



use strict;

use Getopt::Std;
use DBI;



my $common_virtual_domain   = 'domain.tld';
my $db_name              = 'maildb';
my $db_host                = 'localhost';
my $db_user                = 'maildb';
my $db_passwd           = 'maildb';



my %opts; getopts('a:p:n:u:g:m:q:', \%opts);
my $address = $opts{'a'};
unless(defined($address)) {
    usage();
    die("  ");
}
my $domain  = $address;
if($domain =~ /^.+\@([0-9a-z\-]+(\.[0-9a-z\-]+)+)$/) {
    $domain = $1;
} else {
    die("   ");
}
my $passwd  = $opts{'p'};
unless(defined($passwd)) {
    $passwd = pwgen(8, 3);
}
my $name    = $opts{'n'};
unless(defined($name)) {
    $name   = '';
}
my $uid     = $opts{'u'};
unless(defined($uid)) {
    $uid    = 990;
}
my $gid     = $opts{'g'};
unless(defined($gid)) {
    $gid    = 990;
}
my $maildir = $opts{'m'};
unless(defined($maildir)) {
    $maildir = $domain . '/' . $address . '/';
}
my $quota   = $opts{'q'};
unless(defined($quota)) {
    $quota = 0;
}



my $dbh = DBI->connect(
    'DBI:mysql:' . $db_name . ';host=' . $db_host,
    $db_user,
    $db_passwd
);
unless(defined($dbh)) {
    die('   DBI::db->connect(): ' . $DBI::errstr);
}
my $sth_find_last = $dbh->prepare(
    'SELECT address ' .
    'FROM users ' .
    'WHERE address LIKE ? ' .
    'ORDER BY address DESC ' .
    'LIMIT 1'
);
my $sth_update_users = $dbh->prepare(
    'INSERT INTO users ' .
    'VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?)'
);
my $sth_update_virtual = $dbh->prepare(
    'INSERT INTO virtual ' .
    'VALUES (?, ?, ?)'
);



my $usermask;

if(defined($usermask)) {
    my $usermask_regex = $usermask;
       $usermask_regex =~ s/(.*)%(.*)/^($1)(\\d+)($2)\$/;
       $usermask_regex =~ s/\./\\./g;
    unless(defined($sth_find_last->execute($usermask))) {
        die('Can\'t DBI::st->execute(): ' . $dbh->errstr);
    }
    my($db_address) = $sth_find_last->fetchrow_array;
    if($db_address =~ /$usermask_regex/) {
        $address = $1 . ($2 + 1) . $3;
    } else {
        die("!");
    }
}



unless(defined($sth_update_users->execute(
    0,
    $address,
    $passwd,
    crypt($passwd, '$1$' . pwgen(8, 3) . '$'),
    $name,
    $uid,
    $gid,
    $maildir,
    $quota * 1024 * 1024,
    1
))) {
    die('   DBI::st->execute(): ' . $dbh->errstr);
}

my $login;
if($address ne $opts{'a'}) {
    $login = $address; $login =~ s/^(.+)\@$common_virtual_domain$/$1/;
    unless(defined($sth_update_virtual->execute(
        0,
        $opts{'a'},
        $address
    ))) {
        die('   DBI::st->execute(): ' . $dbh->errstr);
    }
} else {
    $login = $opts{'a'};
}



print("  : $opts{'a'}\n");
print(" :        $login\n");
print(":                  $passwd\n");



exit;



sub pwgen {
    my($chars, $type) = @_;
    my $result = '';

    my @charsets = (
        ['a'..'z'],
        ['A'..'Z'],
        ['0'..'9'],
        ['#', ',', qw(! " $ % & ' ( ) * + - . / : ; < = > ? @ [ \ ] ^ _ ` { | } ~)]
    );

    while($chars--) {
        my $charset = $charsets[int(rand($type))];
        my $char;
        while (1) {
            $char = ${$charset}[int(rand(scalar(@{$charset})))];
            if(-1 == index($result, $char)) {
                last;
            }
        }
        $result .= $char;
    }

    return($result);
}



sub usage {
    print(STDERR ": $0 -a <> [-p <>] [-n <>] [-u <uid>] [-g <gid>] [-m <>] [-q <>]\n");
}
